// JawDuino with state machine, updated for SERVO

// To use this code:
// 1. Test the sweep of your servo.  Connect the servo without having it connected to your skull's jaw.  Scroll
// to the bottom of this code to see detailed info.  Also refer to mikkojay's instructions on 
// http://buttonbanger.com/?page_id=137
// 2. Figure out how long your MP3s are and update the various TalkTime values to match.

// Note that I have the LED eyes wired in series so they are controlled by one pin.

// 10/15/2016 - FINAL with timing for skull recordings for Academy of Sciences 2016.

// *** EDIT HERE TO CHANGE THE LENGTH OF THE MP3s ***
// MP3s in seconds (1000 = 1 second)
const long StartTime = 2000;    // Start up time, leave this alone
const long TalkTime1 = 23500;   // Time for mp3 to play.  Change for each mp3  
const long TalkTime2 = 15500;
const long TalkTime3 = 23000;
const long TalkTime4 = 16000;
const long TalkTime5 = 36500;
const long TalkTime6 = 18000;

const long WaitTime = 30000;      // How many seconds to wait after each mp3 plays, before the next one plays.


// MP3 control
int idx = 0;      // mp3 index
int mp3Tot = 7;   // Total number of mp3s + 1 for start state

// Software serial support for Catalex
#include <SoftwareSerial.h>
#define ARDUINO_RX 4// (Optional) should connect to TX of the Serial MP3 Player module
#define ARDUINO_TX A3// Connect ANALOG 3 to RX of the module
SoftwareSerial mySerial(ARDUINO_RX, ARDUINO_TX);

// Catalex commands - these are cut down to just what's needed
static int8_t Send_buf[8] = {0} ;

#define CMD_SEL_DEV 0X09
#define DEV_TF 0X02
#define CMD_PLAY_W_VOL 0X22
#define CMD_STOP_PLAY 0X16

// Servo soft support
#include "SoftRcPulseOut.h"
#define TEST_PIN 11     // Pin 11 set to ground will kick off the servo sweep test
int audio_value = 0;
long lastMsg = 0;
long sleepWindow = 300000; // If 5 minutes go by with no signal, then put the servos to bed
SoftRcPulseOut servo;
volatile boolean servosEnabled = false;
volatile boolean ledsOn = true;
volatile unsigned long currentTime = 0;
volatile unsigned long lastLEDtime = 0;
unsigned long resetWait = 120000; //servos sleep if not changed within this time frame (120 secs)

const int Eyes = A4;            // Pin for the LED eyes

unsigned long StateStartTime=0L;  // Variable for start time
unsigned long StateDuration=0L;   // Variable for duration per state

void setup() {
  Serial.begin(9600);        // Initialize serial communication for debugging
  pinMode(Eyes, OUTPUT);     // Declare eye LEDs as an output

  set_minmax();
  pinMode(TEST_PIN,INPUT);      // Pin 11 will be tester
  digitalWrite(TEST_PIN,HIGH);  // Assign pull-up resistor

  mySerial.begin(9600);     // Set up the soft serial connection for the Catalex
  delay(500);                               // Wait for chip initialization to complete
  sendCommand(CMD_SEL_DEV, DEV_TF);         // Select the TF card  
  delay(200);                               // Wait for 200ms

  StateStartTime=millis();      // Store current time from millis() in StateStartTime.
  StateDuration=StartTime;      // Set initial StateDuration to current StartTime length (established above as a const int)
                                // StateDuration is updated to control length of each state
}  


void loop()
{

  servo_test();
  
  if(millis()-StateStartTime > StateDuration)  // If current time - StateStartTime > StateDuration...
    ChangeState();                             // ...then do ChangeState function.
    
    //Make the skull talk
    audio_updates();          // Read sample from audio input and apply average to buffer
    if(servosEnabled) 
    {
    action();                 // Servos are handled in the action loop
    SoftRcPulseOut::refresh();
    if((millis() - lastMsg) > sleepWindow)
      detach_servos();
    }
   if(millis()-StateStartTime > (StateDuration - WaitTime))   // Turns eyes off after mp3 has stopped
        digitalWrite(Eyes, LOW); 

 }

void ChangeState()                         // Manages the state
{
StateStartTime=millis();                   // Store current time in StateStartTime
  switch (idx)                             // idx variable controls the state
  {
  case 0:
    Serial.println("case START");
    StateDuration=StartTime;               // Set StateDuration to length in StartTime
    
    Serial.println("StateDuration");
    Serial.println (StateDuration);

    sendCommand(CMD_STOP_PLAY, 0X16);
    
    Serial.println(idx);    // Prints value of mp3 to Serial Monitor
    idx++;

    break;

  case 1:
    Serial.println("case 1, mp3 1");
    StateDuration=TalkTime1 + WaitTime;             // Set StateDuration to length in TalkTime

    Serial.println("StateDuration");
    Serial.println (StateDuration);

    digitalWrite(Eyes, HIGH);            // Turn Eyes ON
             
    sendCommand(CMD_PLAY_W_VOL, 0X1E01);   // Play the mp3

    Serial.println(idx);    // Prints value of mp3 to Serial Monitor
    idx++;
      
    break;

  case 2:
    Serial.println("case case 2, mp3 2");
    StateDuration=TalkTime2 + WaitTime;             // Set StateDuration to length in TalkTime

    Serial.println("StateDuration");
    Serial.println (StateDuration);

    digitalWrite(Eyes, HIGH);            // Turn Eyes ON
              
    sendCommand(CMD_PLAY_W_VOL, 0X1E02);   // Play the mp3

    Serial.println(idx);    // Prints value of mp3 to Serial Monitor
    idx++;
    
    break;

  case 3:                            // 
    Serial.println("case case 3, mp3 3");
    StateDuration=TalkTime3 + WaitTime;             // Set StateDuration to length in TalkTime

    Serial.println("StateDuration");
    Serial.println (StateDuration);

    digitalWrite(Eyes, HIGH);            // Turn Eyes ON
              
    sendCommand(CMD_PLAY_W_VOL, 0X1E03);   // Play the mp3

    Serial.println(idx);    // Prints value of mp3 to Serial Monitor
    idx++;
    
    break;

  case 4:
    Serial.println("case case 4, mp3 4");
    StateDuration=TalkTime4 + WaitTime;             // Set StateDuration to length in TalkTime

    Serial.println("StateDuration");
    Serial.println (StateDuration);

    digitalWrite(Eyes, HIGH);            // Turn Eyes ON
              
    sendCommand(CMD_PLAY_W_VOL, 0X1E04);   // Play the mp3

    Serial.println(idx);    // Prints value of mp3 to Serial Monitor
    idx++;
    
    break;

  case 5:
    Serial.println("case case 5, mp3 5");  
    StateDuration=TalkTime5 + WaitTime;             // Set StateDuration to length in TalkTime

    Serial.println("StateDuration");
    Serial.println (StateDuration);

    digitalWrite(Eyes, HIGH);            // Turn Eyes ON
               
    sendCommand(CMD_PLAY_W_VOL, 0X1E05);   // Play the mp3

    Serial.println(idx);    // Prints value of mp3 to Serial Monitor
    idx++;
    
    break;

  case 6:
    Serial.println("case case 6, mp3 6");  
    StateDuration=TalkTime6 + WaitTime;             // Set StateDuration to length in TalkTime

    Serial.println("StateDuration");
    Serial.println (StateDuration);

    digitalWrite(Eyes, HIGH);            // Turn Eyes ON
             
    sendCommand(CMD_PLAY_W_VOL, 0X1E06);   // Play the mp3

    Serial.println(idx);    // Prints value of mp3 to Serial Monitor
    idx++;
    
    break;

  }
  if (idx == mp3Tot) {        // Resets idx
   idx = 0;
   } 
  
}

void sendCommand(int8_t command, int16_t dat)
{
  delay(20);
  Send_buf[0] = 0x7e; //starting byte
  Send_buf[1] = 0xff; //version
  Send_buf[2] = 0x06; //the number of bytes of the command without starting byte and ending byte
  Send_buf[3] = command; //
  Send_buf[4] = 0x00;//0x00 = no feedback, 0x01 = feedback
  Send_buf[5] = (int8_t)(dat >> 8);//datah
  Send_buf[6] = (int8_t)(dat); //datal
  Send_buf[7] = 0xef; //ending byte
  for(uint8_t i=0; i<8; i++)//
  {
    mySerial.write(Send_buf[i]) ;
  }
}

void attach_servos()
{   // attach the pin to the servo object
    servo.attach(A5);
    servosEnabled = true;
}
void detach_servos()
{   // detach the servo objects
    servo.detach();
    servosEnabled = false;
}
void servo_test()
{
  if(digitalRead(TEST_PIN) == HIGH) return;
  attach_servos();
  SoftRcPulseOut::refresh();
  for(int i = 0; i < 360;i++)
  {
    if(i < 180)
      audio_value = i;
    else
      audio_value = 359 - i;
    action();
    for(int i = 0; i < 10; i++)
    {
       delay(1); 
       SoftRcPulseOut::refresh();
    }
  }
  detach_servos();
}
void audio_updates()
{
  audio_value = 0;
  if(analogRead(A0) < 341) audio_value += 60;
  if(analogRead(A1) < 341) audio_value += 60;
  if(analogRead(A2) < 341) audio_value += 60;

  if(audio_value > 0) 
  {
    lastMsg = millis(); //save the time stamp from when we last had some action
    if(!servosEnabled)attach_servos();
  }
}
void action() { 
  if (!servosEnabled) attach_servos();
  servo.write(audio_value);
  SoftRcPulseOut::refresh();
} 
void set_minmax()
{
  // Set the first parameter in the following functions to a number between 0 and 180.
  // I used 92 and 72 in my tests to give about 20 degrees of motion.
  // You may swap the large and small numbers to reverse direction.
  // Just play with them, upload the code, then ground pin 11 to run the sweep test.
  // Be sure to only play with these numbers while the jaw linkage is disconnected,
  // otherwise, you risk hitting mechanical limits and damaging your linkage or servo!
  servo.setMinimumPulse(map(150,0,180,512,2400));   // Mouth closed on my skull
  servo.setMaximumPulse(map(100,0,180,512,2400));   // Amount mouth opens on my skull
}
